# Music Applet
# Copyright (C) 2004-2006 Paul Kuliniewicz <paul@kuliniewicz.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02111-1301, USA.

"""
Various compile-time definitions.

Plugins will be most interested in:
    * VERSION: the version of Music Applet they are running in
    * PKG_DATA_DIR: the directory where Music Applet's data files are
"""

import os

DATA_DIR = "/usr/share"
LIB_DIR = "/usr/lib"
VERSION = "2.4.2"
PACKAGE = "music-applet"
PYTHON_DIR = "/usr/lib/python2.5/site-packages"
PKG_DATA_DIR = "/usr/share/music-applet"
